searchData={"items":[{"type":"module","title":"Phoenix.HTML","doc":"The default building blocks for working with HTML safely\nin Phoenix.\n\nThis library provides three main functionalities:\n\n  * HTML safety\n  * Form handling (with CSRF protection)\n  * A tiny JavaScript library to enhance applications","ref":"Phoenix.HTML.html"},{"type":"module","title":"HTML safety - Phoenix.HTML","doc":"One of the main responsibilities of this package is to\nprovide convenience functions for escaping and marking\nHTML code as safe.\n\nBy default, data output in templates is not considered\nsafe:\n\n     \" %>\n\nwill be shown as:\n\n    &lt;hello&gt;\n\nUser data or data coming from the database is almost never\nconsidered safe. However, in some cases, you may want to tag\nit as safe and show its \"raw\" contents:\n\n     \" %>","ref":"Phoenix.HTML.html#module-html-safety"},{"type":"module","title":"Form handling - Phoenix.HTML","doc":"See `Phoenix.HTML.Form`.","ref":"Phoenix.HTML.html#module-form-handling"},{"type":"module","title":"JavaScript library - Phoenix.HTML","doc":"This project ships with a tiny bit of JavaScript that listens\nto all click events to:\n\n  * Support `data-confirm=\"message\"` attributes, which shows\n    a confirmation modal with the given message\n\n  * Support `data-method=\"patch|post|put|delete\"` attributes,\n    which sends the current click as a PATCH/POST/PUT/DELETE\n    HTTP request. You will need to add `data-to` with the URL\n    and `data-csrf` with the CSRF token value\n\n  * Dispatch a \"phoenix.link.click\" event. You can listen to this\n    event to customize the behaviour above. Returning false from\n    this event will disable `data-method`. Stopping propagation\n    will disable `data-confirm`\n\nTo use the functionality above, you must load `priv/static/phoenix_html.js`\ninto your build tool.\n\n#","ref":"Phoenix.HTML.html#module-javascript-library"},{"type":"module","title":"Overriding the default confirmation behaviour - Phoenix.HTML","doc":"You can override the default implementation by hooking\ninto `phoenix.link.click`. Here is an example:\n\n```javascript\nwindow.addEventListener('phoenix.link.click', function (e) {\n  // Introduce custom behaviour\n  var message = e.target.getAttribute(\"data-prompt\");\n  var answer = e.target.getAttribute(\"data-prompt-answer\");\n  if(message && answer && (answer != window.prompt(message))) {\n    e.preventDefault();\n  }\n}, false);\n```","ref":"Phoenix.HTML.html#module-overriding-the-default-confirmation-behaviour"},{"type":"function","title":"Phoenix.HTML.attributes_escape/1","doc":"Escapes an enumerable of attributes, returning iodata.\n\nThe attributes are rendered in the given order. Note if\na map is given, the key ordering is not guaranteed.\n\nThe keys and values can be of any shape, as long as they\nimplement the `Phoenix.HTML.Safe` protocol. In addition,\nif the key is an atom, it will be \"dasherized\". In other\nwords, `:phx_value_id` will be converted to `phx-value-id`.\n\nFurthermore, the following attributes provide behaviour:\n\n  * `:aria`, `:data`, and `:phx` - they accept a keyword list as\n    value. `data: [confirm: \"are you sure?\"]` is converted to\n    `data-confirm=\"are you sure?\"`.\n\n  * `:class` - it accepts a list of classes as argument. Each\n    element in the list is separated by space. `nil` and `false`\n    elements are discarded. `class: [\"foo\", nil, \"bar\"]` then\n    becomes `class=\"foo bar\"`.\n\n  * `:id` - it is validated raise if a number is given as ID,\n    which is not allowed by the HTML spec and leads to unpredictable\n    behaviour.","ref":"Phoenix.HTML.html#attributes_escape/1"},{"type":"function","title":"Examples - Phoenix.HTML.attributes_escape/1","doc":"iex> safe_to_string attributes_escape(title: \"the title\", id: \"the id\", selected: true)\n    \" title=\\\"the title\\\" id=\\\"the id\\\" selected\"\n\n    iex> safe_to_string attributes_escape(%{data: [confirm: \"Are you sure?\"]})\n    \" data-confirm=\\\"Are you sure?\\\"\"\n\n    iex> safe_to_string attributes_escape(%{phx: [value: [foo: \"bar\"]]})\n    \" phx-value-foo=\\\"bar\\\"\"","ref":"Phoenix.HTML.html#attributes_escape/1-examples"},{"type":"function","title":"Phoenix.HTML.html_escape/1","doc":"Escapes the HTML entities in the given term, returning safe iodata.\n\n    iex> html_escape(\" \")\n    {:safe, [[[] | \"&lt;\"], \"hello\" | \"&gt;\"]}\n\n    iex> html_escape(' ')\n    {:safe, [\"&lt;\", 104, 101, 108, 108, 111, \"&gt;\"]}\n\n    iex> html_escape(1)\n    {:safe, \"1\"}\n\n    iex> html_escape({:safe, \" \"})\n    {:safe, \" \"}","ref":"Phoenix.HTML.html#html_escape/1"},{"type":"function","title":"Phoenix.HTML.javascript_escape/1","doc":"Escapes HTML content to be inserted a JavaScript string.\n\nThis function is useful in JavaScript responses when there is a need\nto escape HTML rendered from other templates, like in the following:\n\n    $(\"#container\").append(\" \");\n\nIt escapes quotes (double and single), double backslashes and others.","ref":"Phoenix.HTML.html#javascript_escape/1"},{"type":"function","title":"Phoenix.HTML.raw/1","doc":"Marks the given content as raw.\n\nThis means any HTML code inside the given\nstring won't be escaped.\n\n    iex> raw(\" \")\n    {:safe, \" \"}\n    iex> raw({:safe, \" \"})\n    {:safe, \" \"}\n    iex> raw(nil)\n    {:safe, \"\"}","ref":"Phoenix.HTML.html#raw/1"},{"type":"function","title":"Phoenix.HTML.safe_to_string/1","doc":"Converts a safe result into a string.\n\nFails if the result is not safe. In such cases, you can\ninvoke `html_escape/1` or `raw/1` accordingly before.\n\nYou can combine `html_escape/1` and `safe_to_string/1`\nto convert a data structure to a escaped string:\n\n    data |> html_escape() |> safe_to_string()","ref":"Phoenix.HTML.html#safe_to_string/1"},{"type":"type","title":"Phoenix.HTML.safe/0","doc":"Guaranteed to be safe","ref":"Phoenix.HTML.html#t:safe/0"},{"type":"type","title":"Phoenix.HTML.unsafe/0","doc":"May be safe or unsafe (i.e. it needs to be converted)","ref":"Phoenix.HTML.html#t:unsafe/0"},{"type":"module","title":"Phoenix.HTML.Engine","doc":"An EEx.Engine that guarantees templates are HTML Safe.","ref":"Phoenix.HTML.Engine.html"},{"type":"function","title":"Phoenix.HTML.Engine.encode_to_iodata!/1","doc":"Encodes the HTML templates to iodata.","ref":"Phoenix.HTML.Engine.html#encode_to_iodata!/1"},{"type":"module","title":"Phoenix.HTML.Form","doc":"Define a `Phoenix.HTML.Form` struct and functions to interact with it.","ref":"Phoenix.HTML.Form.html"},{"type":"module","title":"Access behaviour - Phoenix.HTML.Form","doc":"The `Phoenix.HTML.Form` struct implements the `Access` behaviour.\nWhen you do `form[field]`, it returns a `Phoenix.HTML.FormField`\nstruct with the `id`, `name`, `value`, and `errors` prefilled.\n\nThe field name can be either an atom or a string. If it is an atom,\nit assumes the form keeps both data and errors as atoms. If it is a\nstring, it considers that data and errors are stored as strings for said\nfield. Forms backed by an `Ecto.Changeset` only support atom field names.\n\nIt is possible to \"access\" fields which do not exist in the source data\nstructure. A `Phoenix.HTML.FormField` struct will be dynamically created\nwith some attributes such as `name` and `id` populated.","ref":"Phoenix.HTML.Form.html#module-access-behaviour"},{"type":"function","title":"Phoenix.HTML.Form.__struct__/0","doc":"Defines the Phoenix.HTML.Form struct.\n\nIts fields are:\n\n  * `:source` - the data structure given to `form_for/4` that\n    implements the form data protocol\n\n  * `:impl` - the module with the form data protocol implementation.\n    This is used to avoid multiple protocol dispatches.\n\n  * `:id` - the id to be used when generating input fields\n\n  * `:index` - the index of the struct in the form\n\n  * `:name` - the name to be used when generating input fields\n\n  * `:data` - the field used to store lookup data\n\n  * `:params` - the parameters associated with this form\n\n  * `:hidden` - a keyword list of fields that are required to\n    submit the form behind the scenes as hidden inputs\n\n  * `:options` - a copy of the options given when creating the\n    form via `form_for/4` without any form data specific key\n\n  * `:errors` - a keyword list of errors that are associated with\n    the form","ref":"Phoenix.HTML.Form.html#__struct__/0"},{"type":"function","title":"Phoenix.HTML.Form.checkbox/3","doc":"Generates a checkbox.\n\nThis function is useful for sending boolean values to the server.","ref":"Phoenix.HTML.Form.html#checkbox/3"},{"type":"function","title":"Examples - Phoenix.HTML.Form.checkbox/3","doc":"# Assuming form contains a User schema\n    checkbox(form, :famous)\n    #=>  \n    #=>","ref":"Phoenix.HTML.Form.html#checkbox/3-examples"},{"type":"function","title":"Options - Phoenix.HTML.Form.checkbox/3","doc":"* `:checked_value` - the value to be sent when the checkbox is checked.\n    Defaults to \"true\"\n\n  * `:hidden_input` - controls if this function will generate a hidden input\n    to submit the unchecked value or not. Defaults to \"true\"\n\n  * `:unchecked_value` - the value to be sent when the checkbox is unchecked,\n    Defaults to \"false\"\n\n  * `:value` - the value used to check if a checkbox is checked or unchecked.\n    The default value is extracted from the form data if available\n\nAll other options are forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#checkbox/3-options"},{"type":"function","title":"Hidden fields - Phoenix.HTML.Form.checkbox/3","doc":"Because an unchecked checkbox is not sent to the server, Phoenix\nautomatically generates a hidden field with the unchecked_value\n*before* the checkbox field to ensure the `unchecked_value` is sent\nwhen the checkbox is not marked. Set `hidden_input` to false If you\ndon't want to send values from unchecked checkbox to the server.","ref":"Phoenix.HTML.Form.html#checkbox/3-hidden-fields"},{"type":"function","title":"Phoenix.HTML.Form.color_input/3","doc":"Generates a color input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#color_input/3"},{"type":"function","title":"Phoenix.HTML.Form.date_input/3","doc":"Generates a date input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#date_input/3"},{"type":"function","title":"Phoenix.HTML.Form.date_select/3","doc":"Generates select tags for date.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `date_input/3` instead.\n\nCheck `datetime_select/3` for more information on options and supported values.","ref":"Phoenix.HTML.Form.html#date_select/3"},{"type":"function","title":"Phoenix.HTML.Form.datetime_local_input/3","doc":"Generates a datetime-local input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#datetime_local_input/3"},{"type":"function","title":"Phoenix.HTML.Form.datetime_select/3","doc":"Generates select tags for datetime.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `datetime_local_input/3` instead.","ref":"Phoenix.HTML.Form.html#datetime_select/3"},{"type":"function","title":"Examples - Phoenix.HTML.Form.datetime_select/3","doc":"# Assuming form contains a User schema\n    datetime_select form, :born_at\n    #=>  ...  /\n    #=>  ...  /\n    #=>  ...  —\n    #=>  ...  :\n    #=>  ... \n\nIf you want to include the seconds field (hidden by default), pass `second: []`:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, second: []\n\nIf you want to configure the years range:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, year: [options: 1900..2100]\n\nYou are also able to configure `:month`, `:day`, `:hour`, `:minute` and\n`:second`. All options given to those keys will be forwarded to the\nunderlying select. See `select/4` for more information.\n\nFor example, if you are using Phoenix with Gettext and you want to localize\nthe list of months, you can pass `:options` to the `:month` key:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, month: [\n      options: [\n        {gettext(\"January\"), \"1\"},\n        {gettext(\"February\"), \"2\"},\n        {gettext(\"March\"), \"3\"},\n        {gettext(\"April\"), \"4\"},\n        {gettext(\"May\"), \"5\"},\n        {gettext(\"June\"), \"6\"},\n        {gettext(\"July\"), \"7\"},\n        {gettext(\"August\"), \"8\"},\n        {gettext(\"September\"), \"9\"},\n        {gettext(\"October\"), \"10\"},\n        {gettext(\"November\"), \"11\"},\n        {gettext(\"December\"), \"12\"},\n      ]\n    ]\n\nYou may even provide your own `localized_datetime_select/3` built on top of\n`datetime_select/3`:\n\n    defp localized_datetime_select(form, field, opts \\\\ []) do\n      opts =\n        Keyword.put(opts, :month, options: [\n          {gettext(\"January\"), \"1\"},\n          {gettext(\"February\"), \"2\"},\n          {gettext(\"March\"), \"3\"},\n          {gettext(\"April\"), \"4\"},\n          {gettext(\"May\"), \"5\"},\n          {gettext(\"June\"), \"6\"},\n          {gettext(\"July\"), \"7\"},\n          {gettext(\"August\"), \"8\"},\n          {gettext(\"September\"), \"9\"},\n          {gettext(\"October\"), \"10\"},\n          {gettext(\"November\"), \"11\"},\n          {gettext(\"December\"), \"12\"},\n        ])\n\n      datetime_select(form, field, opts)\n    end","ref":"Phoenix.HTML.Form.html#datetime_select/3-examples"},{"type":"function","title":"Options - Phoenix.HTML.Form.datetime_select/3","doc":"* `:value` - the value used to select a given option.\n    The default value is extracted from the form data if available.\n\n  * `:default` - the default value to use when none was given in\n    `:value` and none is available in the form data\n\n  * `:year`, `:month`, `:day`, `:hour`, `:minute`, `:second` - options passed\n    to the underlying select. See `select/4` for more information.\n    The available values can be given in `:options`.\n\n  * `:builder` - specify how the select can be build. It must be a function\n    that receives a builder that should be invoked with the select name\n    and a set of options. See builder below for more information.","ref":"Phoenix.HTML.Form.html#datetime_select/3-options"},{"type":"function","title":"Builder - Phoenix.HTML.Form.datetime_select/3","doc":"The generated datetime_select can be customized at will by providing a\nbuilder option. Here is an example from EEx:\n\n      %>\n      Date:   /   /  \n      Time:   :  \n     \n\nAlthough we have passed empty lists as options (they are required), you\ncould pass any option there and it would be given to the underlying select\ninput.\n\nIn practice, we recommend you to create your own helper with your default\nbuilder:\n\n    def my_datetime_select(form, field, opts \\\\ []) do\n      builder = fn b ->\n        assigns = %{b: b}\n\n        ~H\"\"\"\n        Date:   /   /  \n        Time:   :  \n        \"\"\"\n      end\n\n      datetime_select(form, field, [builder: builder] ++ opts)\n    end\n\nThen you are able to use your own datetime_select throughout your whole\napplication.","ref":"Phoenix.HTML.Form.html#datetime_select/3-builder"},{"type":"function","title":"Supported date values - Phoenix.HTML.Form.datetime_select/3","doc":"The following values are supported as date:\n\n  * a map containing the `year`, `month` and `day` keys (either as strings or atoms)\n  * a tuple with three elements: `{year, month, day}`\n  * a string in ISO 8601 format\n  * `nil`","ref":"Phoenix.HTML.Form.html#datetime_select/3-supported-date-values"},{"type":"function","title":"Supported time values - Phoenix.HTML.Form.datetime_select/3","doc":"The following values are supported as time:\n\n  * a map containing the `hour` and `minute` keys and an optional `second` key (either as strings or atoms)\n  * a tuple with three elements: `{hour, min, sec}`\n  * a tuple with four elements: `{hour, min, sec, usec}`\n  * `nil`","ref":"Phoenix.HTML.Form.html#datetime_select/3-supported-time-values"},{"type":"function","title":"Phoenix.HTML.Form.email_input/3","doc":"Generates an email input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#email_input/3"},{"type":"function","title":"Phoenix.HTML.Form.file_input/3","doc":"Generates a file input.\n\nIt requires the given form to be configured with `multipart: true`\nwhen invoking `form_for/4`, otherwise it fails with `ArgumentError`.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#file_input/3"},{"type":"function","title":"Phoenix.HTML.Form.form_for/4","doc":"Generates a form tag with a form builder and an anonymous function.\n\n      %>\n      Name:  \n     \n\nForms may be used in two distinct scenarios:\n\n  * with changeset data - when information to populate\n    the form comes from a changeset. The changeset holds\n    rich information, which helps provide conveniences\n\n  * with map data - a simple map of parameters (such as\n    `Plug.Conn.params` can be given as data to the form)\n\nWe will explore all them below.","ref":"Phoenix.HTML.Form.html#form_for/4"},{"type":"function","title":"With changeset data - Phoenix.HTML.Form.form_for/4","doc":"The entry point for defining forms in Phoenix is with\nthe `form_for/4` function. For this example, we will\nuse `Ecto.Changeset`, which integrates nicely with Phoenix\nforms via the `phoenix_ecto` package.\n\nImagine you have the following action in your controller:\n\n    def new(conn, _params) do\n      changeset = User.changeset(%User{})\n      render conn, \"new.html\", changeset: changeset\n    end\n\nwhere `User.changeset/2` is defined as follows:\n\n    def changeset(user, params \\ %{}) do\n      Ecto.Changeset.cast(user, params, [:name, :age])\n    end\n\nNow a `@changeset` assign is available in views which we\ncan pass to the form:\n\n      %>\n       \n        Name:  \n       \n\n       \n        Age:  \n       \n\n       \n     \n\n`form_for/4` receives the `Ecto.Changeset` and converts it\nto a form, which is passed to the function as the argument\n`f`. All the remaining functions in this module receive\nthe form and automatically generate the input fields, often\nby extracting information from the given changeset. For example,\nif the user had a default value for age set, it will\nautomatically show up as selected in the form.\n\n#","ref":"Phoenix.HTML.Form.html#form_for/4-with-changeset-data"},{"type":"function","title":"A note on `:errors` - Phoenix.HTML.Form.form_for/4","doc":"Even if `changeset.errors` is non-empty, errors will not be displayed in a\nform if [the changeset\n`:action`](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-changeset-actions)\nis `nil` or `:ignore`.\n\nThis is useful for things like validation hints on form fields, e.g. an empty\nchangeset for a new form. That changeset isn't valid, but we don't want to\nshow errors until an actual user action has been performed.\n\nFor example, if the user submits and a `Repo.insert/1` is called and fails on\nchangeset validation, the action will be set to `:insert` to show that an\ninsert was attempted, and the presence of that action will cause errors to be\ndisplayed. The same is true for Repo.update/delete.\n\nIf you want to show errors manually you can also set the action yourself,\neither directly on the `Ecto.Changeset` struct field or by using\n`Ecto.Changeset.apply_action/2`. Since the action can be arbitrary, you can\nset it to `:validate` or anything else to avoid giving the impression that a\ndatabase operation has actually been attempted.","ref":"Phoenix.HTML.Form.html#form_for/4-a-note-on-errors"},{"type":"function","title":"With map data - Phoenix.HTML.Form.form_for/4","doc":"`form_for/4` expects as first argument any data structure that\nimplements the `Phoenix.HTML.FormData` protocol. By default,\nPhoenix.HTML implements this protocol for `Map`.\n\nThis is useful when you are creating forms that are not backed\nby any kind of data layer. Let's assume that we're submitting a\nform to the `:new` action in the `FooController`:\n\n      %>\n       \n       \n     \n\nOnce the form is submitted, the form contents will be set directly\nas the parameters root, such as `conn.params[\"contents\"]`. If you\nprefer, you can pass the `:as` option to configure them to be nested:\n\n      %>\n       \n       \n     \n\nIn the example above, all form contents are now set inside `conn.params[\"search\"]`\nthanks to the `[as: :search]` option.","ref":"Phoenix.HTML.Form.html#form_for/4-with-map-data"},{"type":"function","title":"Nested inputs - Phoenix.HTML.Form.form_for/4","doc":"If your data layer supports embedding or nested associations,\nyou can use `inputs_for` to attach nested data to the form.\n\nImagine the following Ecto schemas:\n\n    defmodule User do\n      use Ecto.Schema\n\n      schema \"users\" do\n        field :name\n        embeds_one :permalink, Permalink\n      end\n\n      def changeset(user \\ %User{}, params) do\n        user\n        |> Ecto.Changeset.cast(params, [:name])\n        |> Ecto.Changeset.cast_embed(:permalink)\n      end\n    end\n\n    defmodule Permalink do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :url\n      end\n    end\n\nIn the form, you can now do this:\n\n      %>\n       \n\n        %>\n         \n       \n     \n\nThe default option can be given to populate the fields if none\nis given:\n\n      %>\n       \n     \n\n`inputs_for/4` can be used to work with single entities or\ncollections. When working with collections, `:prepend` and\n`:append` can be used to add entries to the collection\nstored in the changeset.","ref":"Phoenix.HTML.Form.html#form_for/4-nested-inputs"},{"type":"function","title":"CSRF protection - Phoenix.HTML.Form.form_for/4","doc":"CSRF protection is a mechanism to ensure that the user who rendered\nthe form is the one actually submitting it. This module generates a\nCSRF token by default. Your application should check this token on\nthe server to prevent attackers from making requests on your server on\nbehalf of other users. Phoenix checks this token by default.\n\nWhen posting a form with a host in its address, such as \"//host.com/path\"\ninstead of only \"/path\", Phoenix will include the host signature in the\ntoken, and will only validate the token if the accessed host is the same as\nthe host in the token. This is to avoid tokens from leaking to third-party\napplications. If this behaviour is problematic, you can generate a\nnon-host-specific token with `Plug.CSRFProtection.get_csrf_token/0` and\npass it to the form generator via the `:csrf_token` option.","ref":"Phoenix.HTML.Form.html#form_for/4-csrf-protection"},{"type":"function","title":"Options - Phoenix.HTML.Form.form_for/4","doc":"* `:as` - the server side parameter in which all params for this\n    form will be collected (i.e. `as: :user_params` would mean all fields\n    for this form will be accessed as `conn.params.user_params` server\n    side). Automatically inflected when a changeset is given.\n\n  * `:method` - the HTTP method. If the method is not \"get\" nor \"post\",\n    an input tag with name `_method` is generated along-side the form tag.\n    Defaults to \"post\".\n\n  * `:multipart` - when true, sets enctype to \"multipart/form-data\".\n    Required when uploading files.\n\n  * `:csrf_token` - for \"post\" requests, the form tag will automatically\n    include an input tag with name `_csrf_token`. When set to false, this\n    is disabled.\n\n  * `:errors` - use this to manually pass a keyword list of errors to the form\n    (for example from `conn.assigns[:errors]`). This option is only used when a\n    connection is used as the form source and it will make the errors available\n    under `f.errors`.\n\n  * `:id` - the ID of the form attribute. If an ID is given, all form inputs\n    will also be prefixed by the given ID.\n\nAll other options will be passed as HTML attributes, such as `class: \"foo\"`.","ref":"Phoenix.HTML.Form.html#form_for/4-options"},{"type":"function","title":"Phoenix.HTML.Form.hidden_input/3","doc":"Generates a hidden input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#hidden_input/3"},{"type":"function","title":"Phoenix.HTML.Form.hidden_inputs_for/1","doc":"Generates hidden inputs for the given form inputs.\n\nSee `inputs_for/2` and `inputs_for/3`.","ref":"Phoenix.HTML.Form.html#hidden_inputs_for/1"},{"type":"function","title":"Phoenix.HTML.Form.humanize/1","doc":"Converts an attribute/form field into its humanize version.\n\n    iex> humanize(:username)\n    \"Username\"\n    iex> humanize(:created_at)\n    \"Created at\"\n    iex> humanize(\"user_id\")\n    \"User\"","ref":"Phoenix.HTML.Form.html#humanize/1"},{"type":"function","title":"Phoenix.HTML.Form.input_changed?/3","doc":"Receives two forms structs and checks if the given field changed.\n\nThe field will have changed if either its associated value or errors\nchanged. This is mostly used for optimization engines as an extension\nof the `Access` behaviour.","ref":"Phoenix.HTML.Form.html#input_changed?/3"},{"type":"function","title":"Phoenix.HTML.Form.input_id/2","doc":"Returns an id of a corresponding form field.\n\nThe form should either be a `Phoenix.HTML.Form` emitted\nby `form_for` or an atom.","ref":"Phoenix.HTML.Form.html#input_id/2"},{"type":"function","title":"Phoenix.HTML.Form.input_id/3","doc":"Returns an id of a corresponding form field and value attached to it.\n\nUseful for radio buttons and inputs like multiselect checkboxes.","ref":"Phoenix.HTML.Form.html#input_id/3"},{"type":"function","title":"Phoenix.HTML.Form.input_name/2","doc":"Returns a name of a corresponding form field.\n\nThe first argument should either be a `Phoenix.HTML.Form` or an atom.","ref":"Phoenix.HTML.Form.html#input_name/2"},{"type":"function","title":"Examples - Phoenix.HTML.Form.input_name/2","doc":"iex> Phoenix.HTML.Form.input_name(:user, :first_name)\n    \"user[first_name]\"","ref":"Phoenix.HTML.Form.html#input_name/2-examples"},{"type":"function","title":"Phoenix.HTML.Form.input_type/3","doc":"Gets the input type for a given field.\n\nIf the underlying input type is a `:text_field`,\na mapping could be given to further inflect\nthe input type based solely on the field name.\nThe default mapping is:\n\n    %{\"url\"      => :url_input,\n      \"email\"    => :email_input,\n      \"search\"   => :search_input,\n      \"password\" => :password_input}","ref":"Phoenix.HTML.Form.html#input_type/3"},{"type":"function","title":"Phoenix.HTML.Form.input_validations/2","doc":"Returns the HTML validations that would apply to\nthe given field.","ref":"Phoenix.HTML.Form.html#input_validations/2"},{"type":"function","title":"Phoenix.HTML.Form.input_value/2","doc":"Returns a value of a corresponding form field.\n\nThe `form` should either be a `Phoenix.HTML.Form` or an atom.\nThe field is either a string or an atom. If the field is given\nas an atom, it will attempt to look data with atom keys. If\na string, it will look data with string keys.\n\nWhen a form is given, it will look for changes, then\nfallback to parameters, and finally fallback to the default\nstruct/map value.\n\nSince the function looks up parameter values too, there is\nno guarantee that the value will have a certain type. For\nexample, a boolean field will be sent as \"false\" as a\nparameter, and this function will return it as is. If you\nneed to normalize the result of `input_value`, see\n`normalize_value/2`.","ref":"Phoenix.HTML.Form.html#input_value/2"},{"type":"function","title":"Phoenix.HTML.Form.inputs_for/4","doc":"Generate a new form builder for the given parameter in form.\n\nSee `form_for/4` for examples of using this function.","ref":"Phoenix.HTML.Form.html#inputs_for/4"},{"type":"function","title":"Options - Phoenix.HTML.Form.inputs_for/4","doc":"* `:id` - the id to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form id\n\n  * `:as` - the name to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form name\n\n  * `:default` - the value to use if none is available\n\n  * `:prepend` - the values to prepend when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:append` - the values to append when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:skip_hidden` - skip the automatic rendering of hidden\n    fields to allow for more tight control over the generated\n    markup. You can access `form.hidden` to generate them manually\n    within the supplied callback.","ref":"Phoenix.HTML.Form.html#inputs_for/4-options"},{"type":"function","title":"Phoenix.HTML.Form.label/1","doc":"Generates a label tag.\n\nUseful when wrapping another input inside a label.","ref":"Phoenix.HTML.Form.html#label/1"},{"type":"function","title":"Examples - Phoenix.HTML.Form.label/1","doc":"label do\n      radio_button :user, :choice, \"Choice\"\n    end\n    #=>  ... \n\n    label class: \"control-label\" do\n      radio_button :user, :choice, \"Choice\"\n    end\n    #=>  ...","ref":"Phoenix.HTML.Form.html#label/1-examples"},{"type":"function","title":"Phoenix.HTML.Form.label/2","doc":"Generates a label tag for the given field.\n\nThe form should either be a `Phoenix.HTML.Form` emitted\nby `form_for` or an atom.\n\nAll given options are forwarded to the underlying tag.\nA default value is provided for `for` attribute but can\nbe overridden if you pass a value to the `for` option.\nText content would be inferred from `field` if not specified\nas either a function argument or string value in a block.\n\nTo wrap a label around an input, see `label/1`.","ref":"Phoenix.HTML.Form.html#label/2"},{"type":"function","title":"Examples - Phoenix.HTML.Form.label/2","doc":"# Assuming form contains a User schema\n    label(form, :name, \"Name\")\n    #=>  Name \n\n    label(:user, :email, \"Email\")\n    #=>  Email \n\n    label(:user, :email)\n    #=>  Email \n\n    label(:user, :email, class: \"control-label\")\n    #=>  Email \n\n    label :user, :email do\n      \"E-mail Address\"\n    end\n    #=>  E-mail Address \n\n    label :user, :email, \"E-mail Address\", class: \"control-label\"\n    #=>  E-mail Address \n\n    label :user, :email, class: \"control-label\" do\n      \"E-mail Address\"\n    end\n    #=>  E-mail Address","ref":"Phoenix.HTML.Form.html#label/2-examples"},{"type":"function","title":"Phoenix.HTML.Form.label/3","doc":"See `label/2`.","ref":"Phoenix.HTML.Form.html#label/3"},{"type":"function","title":"Phoenix.HTML.Form.label/4","doc":"See `label/2`.","ref":"Phoenix.HTML.Form.html#label/4"},{"type":"function","title":"Phoenix.HTML.Form.multiple_select/4","doc":"Generates a select tag with the given `options`.\n\nValues are expected to be an Enumerable containing two-item tuples\n(like maps and keyword lists) or any Enumerable where the element\nwill be used both as key and value for the generated select.","ref":"Phoenix.HTML.Form.html#multiple_select/4"},{"type":"function","title":"Examples - Phoenix.HTML.Form.multiple_select/4","doc":"# Assuming form contains a User schema\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2])\n    #=>  \n    #=>    Admin \n    #=>    Power User \n    #=>  \n\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2], selected: [1])\n    #=>  \n    #=>    Admin \n    #=>    Power User \n    #=>  \n\nWhen working with structs, associations, and embeds, you will need to tell\nPhoenix how to extract the value out of the collection. For example,\nimagine `user.roles` is a list of `%Role{}` structs. You must call it as:\n\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2],\n                    selected: Enum.map(@user.roles, &(&1.id))\n\nThe `:selected` option will mark the given IDs as selected unless the form\nis being resubmitted. When resubmitted, it uses the form params as values.\n\nWhen used with Ecto, you will typically do a query to retrieve the IDs from\nthe database:\n\n    from r in Role, where: r.id in ^(params[\"roles\"] || [])\n\nAnd then use `Ecto.Changeset.put_assoc/2` to insert the new roles into the user.","ref":"Phoenix.HTML.Form.html#multiple_select/4-examples"},{"type":"function","title":"Options - Phoenix.HTML.Form.multiple_select/4","doc":"* `:selected` - the default options to be marked as selected. The values\n     on this list are ignored in case ids have been set as parameters.\n\nAll other options are forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#multiple_select/4-options"},{"type":"function","title":"Phoenix.HTML.Form.normalize_value/2","doc":"Normalizes an input `value` according to its input `type`.\n\nCertain HTML input values must be cast, or they will have idiosyncracies\nwhen they are rendered. The goal of this function is to encapsulate\nthis logic. In particular:\n\n  * For \"datetime-local\" types, it converts `DateTime` and\n    `NaiveDateTime` to strings without the second precision\n\n  * For \"checkbox\" types, it returns a boolean depending on\n    whether the input is \"true\" or not\n\n  * For \"textarea\", it prefixes a newline to ensure newlines\n    won't be ignored on submission. This requires however\n    that the textarea is rendered with no spaces after its\n    content","ref":"Phoenix.HTML.Form.html#normalize_value/2"},{"type":"function","title":"Phoenix.HTML.Form.number_input/3","doc":"Generates a number input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#number_input/3"},{"type":"function","title":"Phoenix.HTML.Form.options_for_select/2","doc":"Returns options to be used inside a select.\n\nThis is useful when building the select by hand.\nIt expects all options and one or more select values.","ref":"Phoenix.HTML.Form.html#options_for_select/2"},{"type":"function","title":"Examples - Phoenix.HTML.Form.options_for_select/2","doc":"options_for_select([\"Admin\": \"admin\", \"User\": \"user\"], \"admin\")\n    #=>  Admin \n    #=>  User \n\nGroups are also supported:\n\n    options_for_select([\"Europe\": [\"UK\", \"Sweden\", \"France\"], ...], nil)\n    #=>  \n    #=>    UK \n    #=>    Sweden \n    #=>    France \n    #=>","ref":"Phoenix.HTML.Form.html#options_for_select/2-examples"},{"type":"function","title":"Phoenix.HTML.Form.password_input/3","doc":"Generates a password input.\n\nFor security reasons, the form data and parameter values\nare never re-used in `password_input/3`. Pass the value\nexplicitly if you would like to set one.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#password_input/3"},{"type":"function","title":"Phoenix.HTML.Form.radio_button/4","doc":"Generates a radio button.\n\nInvoke this function for each possible value you want\nto be sent to the server.","ref":"Phoenix.HTML.Form.html#radio_button/4"},{"type":"function","title":"Examples - Phoenix.HTML.Form.radio_button/4","doc":"# Assuming form contains a User schema\n    radio_button(form, :role, \"admin\")\n    #=>","ref":"Phoenix.HTML.Form.html#radio_button/4-examples"},{"type":"function","title":"Options - Phoenix.HTML.Form.radio_button/4","doc":"All options are simply forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#radio_button/4-options"},{"type":"function","title":"Phoenix.HTML.Form.range_input/3","doc":"Generates a range input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#range_input/3"},{"type":"function","title":"Phoenix.HTML.Form.reset/2","doc":"Generates a reset input to reset all the form fields to\ntheir original state.\n\nAll options are forwarded to the underlying input tag.","ref":"Phoenix.HTML.Form.html#reset/2"},{"type":"function","title":"Examples - Phoenix.HTML.Form.reset/2","doc":"reset \"Reset\"\n    #=>  \n\n    reset \"Reset\", class: \"btn\"\n    #=>","ref":"Phoenix.HTML.Form.html#reset/2-examples"},{"type":"function","title":"Phoenix.HTML.Form.search_input/3","doc":"Generates a search input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#search_input/3"},{"type":"function","title":"Phoenix.HTML.Form.select/4","doc":"Generates a select tag with the given `options`.\n\n`options` are expected to be an enumerable which will be used to\ngenerate each respective `option`. The enumerable may have:\n\n  * keyword lists - each keyword list is expected to have the keys\n    `:key` and `:value`. Additional keys such as `:disabled` may\n    be given to customize the option.\n\n  * two-item tuples - where the first element is an atom, string or\n    integer to be used as the option label and the second element is\n    an atom, string or integer to be used as the option value\n\n  * atom, string or integer - which will be used as both label and value\n    for the generated select","ref":"Phoenix.HTML.Form.html#select/4"},{"type":"function","title":"Optgroups - Phoenix.HTML.Form.select/4","doc":"If `options` is map or keyword list where the first element is a string,\natom or integer and the second element is a list or a map, it is assumed\nthe key will be wrapped in an ` ` and the value will be used to\ngenerate ` ` nested under the group.","ref":"Phoenix.HTML.Form.html#select/4-optgroups"},{"type":"function","title":"Examples - Phoenix.HTML.Form.select/4","doc":"# Assuming form contains a User schema\n    select(form, :age, 0..120)\n    #=>  \n    #=>    0 \n    #=>   ...\n    #=>    120 \n    #=>  \n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"])\n    #=>  \n    #=>    Admin \n    #=>    User \n    #=>  \n\n    select(form, :role, [[key: \"Admin\", value: \"admin\", disabled: true],\n                         [key: \"User\", value: \"user\"]])\n    #=>  \n    #=>    Admin \n    #=>    User \n    #=>  \n\nYou can also pass a prompt:\n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"], prompt: \"Choose your role\")\n    #=>  \n    #=>    Choose your role \n    #=>    Admin \n    #=>    User \n    #=>  \n\nAnd customize the prompt like any other entry:\n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"], prompt: [key: \"Choose your role\", disabled: true])\n    #=>  \n    #=>    Choose your role \n    #=>    Admin \n    #=>    User \n    #=>  \n\nIf you want to select an option that comes from the database,\nsuch as a manager for a given project, you may write:\n\n    select(form, :manager_id, Enum.map(@managers, &{&1.name, &1.id}))\n    #=>  \n    #=>    Mary Jane \n    #=>    John Doe \n    #=>  \n\nFinally, if the values are a list or a map, we use the keys for\ngrouping:\n\n    select(form, :country, [\"Europe\": [\"UK\", \"Sweden\", \"France\"]], ...)\n    #=>  \n    #=>    \n    #=>      UK \n    #=>      Sweden \n    #=>      France \n    #=>    \n    #=>   ...\n    #=>","ref":"Phoenix.HTML.Form.html#select/4-examples"},{"type":"function","title":"Options - Phoenix.HTML.Form.select/4","doc":"* `:prompt` - an option to include at the top of the options. It may be\n    a string or a keyword list of attributes and the `:key`\n\n  * `:selected` - the default value to use when none was sent as parameter\n\nBe aware that a `:multiple` option will not generate a correctly\nfunctioning multiple select element. Use `multiple_select/4` instead.\n\nAll other options are forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#select/4-options"},{"type":"function","title":"Phoenix.HTML.Form.submit/1","doc":"Generates a submit button to send the form.","ref":"Phoenix.HTML.Form.html#submit/1"},{"type":"function","title":"Examples - Phoenix.HTML.Form.submit/1","doc":"submit do: \"Submit\"\n    #=>  Submit","ref":"Phoenix.HTML.Form.html#submit/1-examples"},{"type":"function","title":"Phoenix.HTML.Form.submit/2","doc":"Generates a submit button to send the form.\n\nAll options are forwarded to the underlying button tag.\nWhen called with a `do:` block, the button tag options\ncome first.","ref":"Phoenix.HTML.Form.html#submit/2"},{"type":"function","title":"Examples - Phoenix.HTML.Form.submit/2","doc":"submit \"Submit\"\n    #=>  Submit \n\n    submit \"Submit\", class: \"btn\"\n    #=>  Submit \n\n    submit [class: \"btn\"], do: \"Submit\"\n    #=>  Submit","ref":"Phoenix.HTML.Form.html#submit/2-examples"},{"type":"function","title":"Phoenix.HTML.Form.telephone_input/3","doc":"Generates a telephone input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#telephone_input/3"},{"type":"function","title":"Phoenix.HTML.Form.text_input/3","doc":"Generates a text input.\n\nThe form should either be a `Phoenix.HTML.Form` emitted\nby `form_for` or an atom.\n\nAll given options are forwarded to the underlying input,\ndefault values are provided for id, name and value if\npossible.","ref":"Phoenix.HTML.Form.html#text_input/3"},{"type":"function","title":"Examples - Phoenix.HTML.Form.text_input/3","doc":"# Assuming form contains a User schema\n    text_input(form, :name)\n    #=>  \n\n    text_input(:user, :name)\n    #=>","ref":"Phoenix.HTML.Form.html#text_input/3-examples"},{"type":"function","title":"Phoenix.HTML.Form.textarea/3","doc":"Generates a textarea input.\n\nAll given options are forwarded to the underlying input,\ndefault values are provided for id, name and textarea\ncontent if possible.","ref":"Phoenix.HTML.Form.html#textarea/3"},{"type":"function","title":"Examples - Phoenix.HTML.Form.textarea/3","doc":"# Assuming form contains a User schema\n    textarea(form, :description)\n    #=>","ref":"Phoenix.HTML.Form.html#textarea/3-examples"},{"type":"function","title":"New lines - Phoenix.HTML.Form.textarea/3","doc":"Notice the generated textarea includes a new line after\nthe opening tag. This is because the HTML spec says new\nlines after tags must be ignored, and all major browser\nimplementations do that.\n\nTherefore, in order to avoid new lines provided by the user\nfrom being ignored when the form is resubmitted, we\nautomatically add a new line before the text area\nvalue.","ref":"Phoenix.HTML.Form.html#textarea/3-new-lines"},{"type":"function","title":"Phoenix.HTML.Form.time_input/3","doc":"Generates a time input.","ref":"Phoenix.HTML.Form.html#time_input/3"},{"type":"function","title":"Options - Phoenix.HTML.Form.time_input/3","doc":"* `:precision` - Allowed values: `:minute`, `:second`, `:millisecond`.\n    Defaults to `:minute`.\n\nAll other options are forwarded. See `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#time_input/3-options"},{"type":"function","title":"Examples - Phoenix.HTML.Form.time_input/3","doc":"time_input form, :time\n    #=>  \n\n    time_input form, :time, precision: :second\n    #=>  \n\n    time_input form, :time, precision: :millisecond\n    #=>","ref":"Phoenix.HTML.Form.html#time_input/3-examples"},{"type":"function","title":"Phoenix.HTML.Form.time_select/3","doc":"Generates select tags for time.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `time_input/3` instead.\n\nCheck `datetime_select/3` for more information on options and supported values.","ref":"Phoenix.HTML.Form.html#time_select/3"},{"type":"function","title":"Phoenix.HTML.Form.url_input/3","doc":"Generates an url input.\n\nSee `text_input/3` for example and docs.","ref":"Phoenix.HTML.Form.html#url_input/3"},{"type":"type","title":"Phoenix.HTML.Form.field/0","doc":"","ref":"Phoenix.HTML.Form.html#t:field/0"},{"type":"type","title":"Phoenix.HTML.Form.t/0","doc":"","ref":"Phoenix.HTML.Form.html#t:t/0"},{"type":"protocol","title":"Phoenix.HTML.FormData","doc":"Converts a data structure into a [`Phoenix.HTML.Form`](`t:Phoenix.HTML.Form.t/0`) struct.","ref":"Phoenix.HTML.FormData.html"},{"type":"protocol","title":"Ecto integration - Phoenix.HTML.FormData","doc":"Phoenix provides integration of forms with Ecto changesets and data\nstructures via the [phoenix_ecto](https://hex.pm/packages/phoenix_ecto) package.\nIf a project was generated without Ecto support that dependency will need to be\nmanually added.","ref":"Phoenix.HTML.FormData.html#module-ecto-integration"},{"type":"function","title":"Phoenix.HTML.FormData.input_type/3","doc":"Receives the given field and returns its input type (:text_input,\n:select, etc). Returns `nil` if the type is unknown.","ref":"Phoenix.HTML.FormData.html#input_type/3"},{"type":"function","title":"Phoenix.HTML.FormData.input_validations/3","doc":"Returns the HTML5 validations that would apply to\nthe given field.","ref":"Phoenix.HTML.FormData.html#input_validations/3"},{"type":"function","title":"Phoenix.HTML.FormData.input_value/3","doc":"Returns the value for the given field.","ref":"Phoenix.HTML.FormData.html#input_value/3"},{"type":"function","title":"Phoenix.HTML.FormData.to_form/2","doc":"Converts a data structure into a [`Phoenix.HTML.Form`](`t:Phoenix.HTML.Form.t/0`) struct.\n\nThe options have their meaning defined by the underlying\nimplementation but all shared options below are expected to\nbe implemented. All remaining options must be stored in the\nreturned struct.","ref":"Phoenix.HTML.FormData.html#to_form/2"},{"type":"function","title":"Shared options - Phoenix.HTML.FormData.to_form/2","doc":"* `:as` - the value to be used as the form name\n\n  * `:id` - the ID of the form attribute. All form inputs will\n    be prefixed by the given ID","ref":"Phoenix.HTML.FormData.html#to_form/2-shared-options"},{"type":"function","title":"Phoenix.HTML.FormData.to_form/4","doc":"Converts the field in the given form based on the data structure\ninto a list of [`Phoenix.HTML.Form`](`t:Phoenix.HTML.Form.t/0`) structs.\n\nThe options have their meaning defined by the underlying\nimplementation but all shared options below are expected to\nbe implemented. All remaining options must be stored in the\nreturned struct.","ref":"Phoenix.HTML.FormData.html#to_form/4"},{"type":"function","title":"Shared Options - Phoenix.HTML.FormData.to_form/4","doc":"* `:id` - the id to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form id\n\n  * `:as` - the name to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form name\n\n  * `:default` - the value to use if none is available\n\n  * `:prepend` - the values to prepend when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:append` - the values to append when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.","ref":"Phoenix.HTML.FormData.html#to_form/4-shared-options"},{"type":"type","title":"Phoenix.HTML.FormData.t/0","doc":"All the types that implement this protocol.","ref":"Phoenix.HTML.FormData.html#t:t/0"},{"type":"module","title":"Phoenix.HTML.FormField","doc":"The struct returned by `form[field]`.\n\nIt has the following fields:\n\n  * `:errors` - a list of errors belonging to the field\n  * `:field` - the field name as an atom or a string\n  * `:form` - the parent `form` struct\n  * `:id` - the `id` to be used as form input as a string\n  * `:name` - the `name` to be used as form input as a string\n  * `:value` - the value for the input","ref":"Phoenix.HTML.FormField.html"},{"type":"module","title":"Phoenix.HTML.Link","doc":"Conveniences for working with links and URLs in HTML.","ref":"Phoenix.HTML.Link.html"},{"type":"function","title":"Phoenix.HTML.Link.button/2","doc":"Generates a button tag that uses the Javascript function handleClick()\n(see phoenix_html.js) to submit the form data.\n\nUseful to ensure that links that change data are not triggered by\nsearch engines and other spidering software.","ref":"Phoenix.HTML.Link.html#button/2"},{"type":"function","title":"Examples - Phoenix.HTML.Link.button/2","doc":"button(\"hello\", to: \"/world\")\n    #=>  hello \n\n    button(\"hello\", to: \"/world\", method: :get, class: \"btn\")\n    #=>  hello","ref":"Phoenix.HTML.Link.html#button/2-examples"},{"type":"function","title":"Options - Phoenix.HTML.Link.button/2","doc":"* `:to` - the page to link to. This option is required\n\n  * `:method` - the method to use with the button. Defaults to :post.\n\nAll other options are forwarded to the underlying button input.\n\nWhen the `:method` is set to `:get` and the `:to` URL contains query\nparameters the generated form element will strip the parameters in accordance\nwith the [W3C](https://www.w3.org/TR/html401/interact/forms.html#h-17.13.3.4)\nform specification.","ref":"Phoenix.HTML.Link.html#button/2-options"},{"type":"function","title":"Data attributes - Phoenix.HTML.Link.button/2","doc":"Data attributes are added as a keyword list passed to the\n`data` key. The following data attributes are supported:\n\n  * `data-confirm` - shows a confirmation prompt before generating and\n    submitting the form.","ref":"Phoenix.HTML.Link.html#button/2-data-attributes"},{"type":"function","title":"Phoenix.HTML.Link.link/2","doc":"Generates a link to the given URL.","ref":"Phoenix.HTML.Link.html#link/2"},{"type":"function","title":"Examples - Phoenix.HTML.Link.link/2","doc":"link(\"hello\", to: \"/world\")\n    #=>  hello \n\n    link(\"hello\", to: URI.parse(\"https://elixir-lang.org\"))\n    #=>  hello \n\n    link(\" \", to: \"/world\")\n    #=>  &lt;hello&gt; \n\n    link(\" \", to: \"/world\", class: \"btn\")\n    #=>  &lt;hello&gt; \n\n    link(\"delete\", to: \"/the_world\", data: [confirm: \"Really?\"])\n    #=>  delete \n\n    # If you supply a method other than `:get`:\n    link(\"delete\", to: \"/everything\", method: :delete)\n    #=>  delete \n\n    # You can use a `do ... end` block too:\n    link to: \"/hello\" do\n      \"world\"\n    end\n    #=>  world","ref":"Phoenix.HTML.Link.html#link/2-examples"},{"type":"function","title":"Options - Phoenix.HTML.Link.link/2","doc":"* `:to` - the page to link to. This option is required\n\n  * `:method` - the method to use with the link. In case the\n    method is not `:get`, the link is generated inside the form\n    which sets the proper information. In order to submit the\n    form, JavaScript must be enabled\n\n  * `:csrf_token` - a custom token to use for links with a method\n    other than `:get`.\n\nAll other options are forwarded to the underlying ` ` tag.","ref":"Phoenix.HTML.Link.html#link/2-options"},{"type":"function","title":"Data attributes - Phoenix.HTML.Link.link/2","doc":"Data attributes are added as a keyword list passed to the `data` key.\nThe following data attributes are supported:\n\n  * `data-confirm` - shows a confirmation prompt before\n    generating and submitting the form when `:method`\n    is not `:get`.","ref":"Phoenix.HTML.Link.html#link/2-data-attributes"},{"type":"function","title":"CSRF Protection - Phoenix.HTML.Link.link/2","doc":"By default, CSRF tokens are generated through `Plug.CSRFProtection`.","ref":"Phoenix.HTML.Link.html#link/2-csrf-protection"},{"type":"protocol","title":"Phoenix.HTML.Safe","doc":"Defines the HTML safe protocol.\n\nIn order to promote HTML safety, Phoenix templates\ndo not use `Kernel.to_string/1` to convert data types to\nstrings in templates. Instead, Phoenix uses this\nprotocol which must be implemented by data structures\nand guarantee that a HTML safe representation is returned.\n\nFurthermore, this protocol relies on iodata, which provides\nbetter performance when sending or streaming data to the client.","ref":"Phoenix.HTML.Safe.html"},{"type":"function","title":"Phoenix.HTML.Safe.to_iodata/1","doc":"","ref":"Phoenix.HTML.Safe.html#to_iodata/1"},{"type":"type","title":"Phoenix.HTML.Safe.t/0","doc":"All the types that implement this protocol.","ref":"Phoenix.HTML.Safe.html#t:t/0"},{"type":"module","title":"Phoenix.HTML.Tag","doc":"Helpers related to producing HTML tags within templates.\n\n> Note: with the addition of the HEEx template engine to\n> Phoenix applications, the functions in this module have\n> lost a bit of relevance and must only be used in special\n> circumstances.\n>\n> Whenever possible, prefer to use the HEEx template engine\n> instead of the functions here. For example, instead of:\n>\n>      \n>       Hello\n>      \n>\n> Do:\n>\n>      \n>       Hello\n>      \n\n> Note: the examples in this module use `safe_to_string/1`\n> imported from `Phoenix.HTML` for readability.","ref":"Phoenix.HTML.Tag.html"},{"type":"function","title":"Phoenix.HTML.Tag.content_tag/2","doc":"Creates an HTML tag with given name, content, and attributes.\n\nSee `Phoenix.HTML.Tag.tag/2` for more information and examples.\n\n    iex> safe_to_string content_tag(:p, \"Hello\")\n    \" Hello \"\n\n    iex> safe_to_string content_tag(:p, \" \", class: \"test\")\n    \" &lt;Hello&gt; \"\n\n    iex> safe_to_string(content_tag :p, class: \"test\" do\n    ...>   \"Hello\"\n    ...> end)\n    \" Hello \"\n\n    iex> safe_to_string content_tag(:option, \"Display Value\", [{:data, [foo: \"bar\"]}, value: \"value\"])\n    \" Display Value \"","ref":"Phoenix.HTML.Tag.html#content_tag/2"},{"type":"function","title":"Phoenix.HTML.Tag.content_tag/3","doc":"","ref":"Phoenix.HTML.Tag.html#content_tag/3"},{"type":"function","title":"Phoenix.HTML.Tag.csrf_input_tag/2","doc":"Generates a hidden input tag with a CSRF token.\n\nThis could be used when writing a form without the use of tag\nhelpers like `form_tag/3` or `form_for/4`, while maintaining\nCSRF protection.\n\nThe `to` argument should be the same as the form action.","ref":"Phoenix.HTML.Tag.html#csrf_input_tag/2"},{"type":"function","title":"Example - Phoenix.HTML.Tag.csrf_input_tag/2","doc":"etc.\n     \n\nAdditional options to the tag can be given.","ref":"Phoenix.HTML.Tag.html#csrf_input_tag/2-example"},{"type":"function","title":"Phoenix.HTML.Tag.csrf_meta_tag/1","doc":"Generates a meta tag with CSRF information.\n\nAdditional options to the tag can be given.","ref":"Phoenix.HTML.Tag.html#csrf_meta_tag/1"},{"type":"function","title":"Phoenix.HTML.Tag.csrf_token_value/1","doc":"Returns the `csrf_token` value to be used by forms, meta tags, etc.\n\nBy default, CSRF tokens are generated through `Plug.CSRFProtection`\nwhich is capable of generating a separate token per host. Therefore\nit is recommended to pass the `URI` of the destination as argument.\nIf none is given `%URI{host: nil}` is used, which implies a local\nrequest is being done.","ref":"Phoenix.HTML.Tag.html#csrf_token_value/1"},{"type":"function","title":"Phoenix.HTML.Tag.form_tag/2","doc":"Generates a form tag.\n\nThis function generates the ` ` tag without its closing part.\nCheck `form_tag/3` for generating an enclosing tag.","ref":"Phoenix.HTML.Tag.html#form_tag/2"},{"type":"function","title":"Examples - Phoenix.HTML.Tag.form_tag/2","doc":"form_tag(\"/hello\")\n     \n\n    form_tag(\"/hello\", method: :get)","ref":"Phoenix.HTML.Tag.html#form_tag/2-examples"},{"type":"function","title":"Options - Phoenix.HTML.Tag.form_tag/2","doc":"* `:method` - the HTTP method. If the method is not \"get\" nor \"post\",\n    an input tag with name `_method` is generated along-side the form tag.\n    Defaults to \"post\".\n\n  * `:multipart` - when true, sets enctype to \"multipart/form-data\".\n    Required when uploading files\n\n  * `:csrf_token` - for \"post\" requests, the form tag will automatically\n    include an input tag with name `_csrf_token`. When set to false, this\n    is disabled\n\nAll other options are passed to the underlying HTML tag.","ref":"Phoenix.HTML.Tag.html#form_tag/2-options"},{"type":"function","title":"CSRF Protection - Phoenix.HTML.Tag.form_tag/2","doc":"By default, CSRF tokens are generated through `Plug.CSRFProtection`.","ref":"Phoenix.HTML.Tag.html#form_tag/2-csrf-protection"},{"type":"function","title":"Phoenix.HTML.Tag.form_tag/3","doc":"Generates a form tag with the given contents.","ref":"Phoenix.HTML.Tag.html#form_tag/3"},{"type":"function","title":"Examples - Phoenix.HTML.Tag.form_tag/3","doc":"form_tag(\"/hello\", method: \"get\") do\n      \"Hello\"\n    end\n     ...Hello...","ref":"Phoenix.HTML.Tag.html#form_tag/3-examples"},{"type":"function","title":"Phoenix.HTML.Tag.img_tag/2","doc":"Generates an img tag with a src.","ref":"Phoenix.HTML.Tag.html#img_tag/2"},{"type":"function","title":"Examples - Phoenix.HTML.Tag.img_tag/2","doc":"img_tag(user.photo_path)\n     \n\n    img_tag(user.photo, class: \"image\")\n     \n\nTo generate a path to an image hosted in your application \"priv/static\",\nwith the `@conn` endpoint, use `static_path/2` to get a URL with\ncache control parameters:\n\n    img_tag(Routes.static_path(@conn, \"/logo.png\"))\n     \n\nFor responsive images, pass a map, list or string through `:srcset`.\n\n    img_tag(\"/logo.png\", srcset: %{\"/logo.png\" => \"1x\", \"/logo-2x.png\" => \"2x\"})\n     \n\n    img_tag(\"/logo.png\", srcset: [\"/logo.png\", {\"/logo-2x.png\", \"2x\"}])","ref":"Phoenix.HTML.Tag.html#img_tag/2-examples"},{"type":"function","title":"Phoenix.HTML.Tag.tag/1","doc":"Creates an HTML tag with the given name and options.\n\n    iex> safe_to_string tag(:br)\n    \" \"\n    iex> safe_to_string tag(:input, type: \"text\", name: \"user_id\")\n    \" \"","ref":"Phoenix.HTML.Tag.html#tag/1"},{"type":"function","title":"Data attributes - Phoenix.HTML.Tag.tag/1","doc":"In order to add custom data attributes you need to pass\na tuple containing :data atom and a keyword list\nwith data attributes' names and values as the first element\nin the tag's attributes keyword list:\n\n    iex> safe_to_string tag(:input, [data: [foo: \"bar\"], id: \"some_id\"])\n    \" \"","ref":"Phoenix.HTML.Tag.html#tag/1-data-attributes"},{"type":"function","title":"Boolean values - Phoenix.HTML.Tag.tag/1","doc":"In case an attribute contains a boolean value, its key\nis repeated when it is true, as expected in HTML, or\nthe attribute is completely removed if it is false:\n\n    iex> safe_to_string tag(:audio, autoplay: \"autoplay\")\n    \" \"\n    iex> safe_to_string tag(:audio, autoplay: true)\n    \" \"\n    iex> safe_to_string tag(:audio, autoplay: false)\n    \" \"\n\nIf you want the boolean attribute to be sent as is,\nyou can explicitly convert it to a string before.","ref":"Phoenix.HTML.Tag.html#tag/1-boolean-values"},{"type":"function","title":"Phoenix.HTML.Tag.tag/2","doc":"","ref":"Phoenix.HTML.Tag.html#tag/2"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v3.3.2 (2023-08-10) - Changelog","doc":"* Enhancements\n  * Address deprecations in Elixir v1.16+\n\n* Deprecations\n  * Deprecate `inputs_for/2` and `inputs_for/3` (without anonymous functions)","ref":"changelog.html#v3-3-2-2023-08-10"},{"type":"extras","title":"v3.3.1 (2023-02-27) - Changelog","doc":"* Bug fix\n  * Set display to none on generated forms\n  * Warn for maps with atom keys","ref":"changelog.html#v3-3-1-2023-02-27"},{"type":"extras","title":"v3.3.0 (2023-02-10) - Changelog","doc":"* Enhancements\n  * Support deeply nested class lists\n  * Implement Phoenix.HTML.Safe for URI\n  * Implement Phoenix.HTML.FormData for Map\n\n* Bug fix\n  * Generate unique IDs for checkboxes based on the value\n  * Use artificial button click instead of `form.submit` in JavaScript to trigger all relevant events\n  * Fix a bug where nil/false/true attributes in `aria`/`data`/`phx` would emit empty or literal values, such as `\"true\"` and `\"false\"`. This release aligns them with all other attributes so both `nil` and `false` emit nothing. `true` emits the attribute with no value.\n\n* Deprecations\n  * `Phoenix.HTML.Tag.attributes_escape/1` is deprecated in favor of `Phoenix.HTML.attributes_escape/1`","ref":"changelog.html#v3-3-0-2023-02-10"},{"type":"extras","title":"v3.2.0 (2021-12-18) - Changelog","doc":"* Enhancements\n  * Raise if the `id` attribute is set to a number. This is actually an invalid value according to the HTML spec and it can lead to problematic client behaviour, especially in LiveView and other client frameworks.\n  * Allow `phx` attributes to be nested, similar to `aria` and `data` attributes\n  * Allow hidden fields in forms to be a list of values","ref":"changelog.html#v3-2-0-2021-12-18"},{"type":"extras","title":"v3.1.0 (2021-10-23) - Changelog","doc":"* Bug fix\n  * Do not submit data-method links if default has been prevented\n* Deprecations\n  * Deprecate `~E` and `Phoenix.HTML.Tag.attributes_escape/1`\n  * Remove deprecated `Phoenix.HTML.Link.link/1`","ref":"changelog.html#v3-1-0-2021-10-23"},{"type":"extras","title":"v3.0.4 (2021-09-23) - Changelog","doc":"* Bug fix\n  * Ensure `class={@class}` in HEEx templates and `:class` attribute in `content_tag` are properly escaped against XSS","ref":"changelog.html#v3-0-4-2021-09-23"},{"type":"extras","title":"v3.0.3 (2021-09-04) - Changelog","doc":"* Bug fix\n  * Fix sorting of attributes in `tag`/`content_tag`","ref":"changelog.html#v3-0-3-2021-09-04"},{"type":"extras","title":"v3.0.2 (2021-08-19) - Changelog","doc":"* Enhancements\n  * Support maps on `Phoenix.HTML.Tag.attributes_escape/1`","ref":"changelog.html#v3-0-2-2021-08-19"},{"type":"extras","title":"v3.0.1 (2021-08-14) - Changelog","doc":"* Enhancements\n  * Add `Phoenix.HTML.Tag.csrf_input_tag/2`","ref":"changelog.html#v3-0-1-2021-08-14"},{"type":"extras","title":"v3.0.0 (2021-08-06) - Changelog","doc":"* Enhancements\n  * Allow extra html attributes on the `:prompt` option in `select`\n  * Make `Plug` an optional dependency\n  * Prefix form id on inputs when it is given to `form_for/3`\n  * Allow `%URI{}` to be passed to `link/2` and `button/2` as `:to`\n  * Expose `Phoenix.HTML.Tag.csrf_token_value/1`\n  * Add `Phoenix.HTML.Tag.attributes_escape/1`\n\n* Bug fixes\n  * Honor the `form` attribute when creating hidden checkbox input\n  * Use `to_iso8601` as the standard implementation for safe dates and times\n\n* Deprecations\n  * `form_for` without an anonymous function has been deprecated. v3.0 has deprecated the usage, v3.1 will emit warnings, and v3.2 will fully remove the functionality\n\n* Backwards incompatible changes\n  * Strings given as attributes keys in `tag` and `content_tag` are now emitted as is (without being dasherized) and are also HTML escaped\n  * Prefix form id on inputs when it is given to `form_for/3`\n  * By default dates and times will format to the `to_iso8601` functions provided by their implementation\n  * Do not include `csrf-param` and `method-param` in generated `csrf_meta_tag`\n  * Remove deprecated `escape_javascript` in favor of `javascript_escape`\n  * Remove deprecated `field_value` in favor of `input_value`\n  * Remove deprecated `field_name` in favor of `input_name`\n  * Remove deprecated `field_id` in favor of `input_id`","ref":"changelog.html#v3-0-0-2021-08-06"},{"type":"extras","title":"v2.14.3 (2020-12-12) - Changelog","doc":"* Bug fixes\n  * Fix warnings on Elixir v1.12","ref":"changelog.html#v2-14-3-2020-12-12"},{"type":"extras","title":"v2.14.2 (2020-04-30) - Changelog","doc":"* Deprecations\n  * Deprecate `Phoenix`-specific assigns `:view_module` and `:view_template`","ref":"changelog.html#v2-14-2-2020-04-30"},{"type":"extras","title":"v2.14.1 (2020-03-20) - Changelog","doc":"* Enhancements\n  * Add `Phoenix.HTML.Form.options_for_select/2`\n  * Add `Phoenix.HTML.Form.inputs_for/3`\n\n* Bug fixes\n  * Disable hidden input for disabled checkboxes","ref":"changelog.html#v2-14-1-2020-03-20"},{"type":"extras","title":"v2.14.0 (2020-01-28) - Changelog","doc":"* Enhancements\n  * Remove enforce_utf8 workaround on forms as it is no longer required by browser\n  * Remove support tuple-based date/time with microseconds calendar types\n  * Allow strings as first element in `content_tag`\n  * Add `:srcset` support to `img_tag`\n  * Allow `inputs_for` to skip hidden fields","ref":"changelog.html#v2-14-0-2020-01-28"},{"type":"extras","title":"v2.13.4 (2020-01-28) - Changelog","doc":"* Bug fixes\n  * Fix invalid :line in Elixir v1.10.0","ref":"changelog.html#v2-13-4-2020-01-28"},{"type":"extras","title":"v2.13.3 (2019-05-31) - Changelog","doc":"* Enhancements\n  * Add atom support to FormData\n\n* Bug fixes\n  * Keep proper line numbers on .eex templates for proper coverage","ref":"changelog.html#v2-13-3-2019-05-31"},{"type":"extras","title":"v2.13.2 (2019-03-29) - Changelog","doc":"* Bug fixes\n  * Stop event propagation when confirm dialog is canceled","ref":"changelog.html#v2-13-2-2019-03-29"},{"type":"extras","title":"v2.13.1 (2019-01-05) - Changelog","doc":"* Enhancements\n  * Allow safe content to be given to label\n  * Also escale template literals in `javascript_escape/1`\n\n* Bug fixes\n  * Fix deprecation warnings to point to the correct alternative","ref":"changelog.html#v2-13-1-2019-01-05"},{"type":"extras","title":"v2.13.0 (2018-12-09) - Changelog","doc":"* Enhancements\n  * Require Elixir v1.5+ for more efficient template compilation/rendering\n  * Add `Phoenix.HTML.Engine.encode_to_iodata!/1`\n  * Add `Phoenix.HTML.Form.form_for/3` that works without an anonymous function\n\n* Deprecations\n  * Deprecate `Phoenix.HTML.escape_javascript/1` in favor of `Phoenix.HTML.javascript_escape/1` for consistency","ref":"changelog.html#v2-13-0-2018-12-09"},{"type":"extras","title":"v2.12.0 (2018-08-06) - Changelog","doc":"* Enhancements\n  * Configurable and extendable data-confirm behaviour\n  * Allow data-confirm with submit buttons\n  * Support ISO 8601 formatted strings for date and time values\n\n* Bug fixes\n  * Provide a default id of the field name for `@conn` based forms","ref":"changelog.html#v2-12-0-2018-08-06"},{"type":"extras","title":"v2.11.2 (2018-04-13) - Changelog","doc":"* Enhancements\n  * Support custom precision on time input\n\n* Bug fixes\n  * Do not raise when `:` is part of a path on link/button attributes","ref":"changelog.html#v2-11-2-2018-04-13"},{"type":"extras","title":"v2.11.1 (2018-03-20) - Changelog","doc":"* Enhancements\n  * Add `label/1`\n  * Copy the target attribute of the link in the generated JS form\n\n* Bug fixes\n  * Support any value that is html escapable in `radio_button`","ref":"changelog.html#v2-11-1-2018-03-20"},{"type":"extras","title":"v2.11.0 (2018-03-09) - Changelog","doc":"* Enhancements\n  * Add date, datetime-local and time input types\n  * Enable string keys to be usable with forms\n  * Support carriage return in `text_to_html`\n  * Add support for HTML5 boolean attributes to `content_tag` and `tag`\n  * Improve performance by relying on `html_safe_to_iodata/1`\n  * Protect against CSRF tokens leaking across hosts when the POST URL is dynamic\n  * Require `to` attribute in links and buttons to explicitly pass protocols as a separate option for safety reasons\n\n* Bug fixes\n  * Guarantee `input_name/2` always returns strings\n  * Improve handling of uncommon whitespace and null in `escape_javascript`\n  * Escape value attribute so it is never treated as a boolean\n\n* Backwards incompatible changes\n  * The :csrf_token_generator configuration in the Phoenix.HTML app no longer works due to the improved security mechanisms","ref":"changelog.html#v2-11-0-2018-03-09"},{"type":"extras","title":"v2.10.5 (2017-11-08) - Changelog","doc":"* Enhancements\n  * Do not require the :as option in form_for","ref":"changelog.html#v2-10-5-2017-11-08"},{"type":"extras","title":"v2.10.4 (2017-08-15) - Changelog","doc":"* Bug fixes\n  * Fix formatting of days in datetime_builder","ref":"changelog.html#v2-10-4-2017-08-15"},{"type":"extras","title":"v2.10.3 (2017-07-30) - Changelog","doc":"* Enhancements\n  * Allow specifying a custom CSRF token generator\n\n* Bug fixes\n  * Do not submit `method: :get` in buttons as \"post\"","ref":"changelog.html#v2-10-3-2017-07-30"},{"type":"extras","title":"v2.10.2 (2017-07-24) - Changelog","doc":"* Bug fixes\n  * Traverse DOM elements up when handling data-method","ref":"changelog.html#v2-10-2-2017-07-24"},{"type":"extras","title":"v2.10.1 (2017-07-22) - Changelog","doc":"* Bug fixes\n  * Only generate CSRF token if necessary","ref":"changelog.html#v2-10-1-2017-07-22"},{"type":"extras","title":"v2.10.0 (2017-07-21) - Changelog","doc":"* Enhancements\n  * Support custom attributes in options in select\n\n* Bug fixes\n  * Accept non-binary values in textarea's content\n  * Allow nested forms on the javascript side. This means `link` and `button` no longer generate a child form such as the `:form` option has no effect and \"data-submit=parent\" is no longer supported. Instead \"data-to\" and \"data-method\" are set on the entities and the form is generated on the javascript side of things","ref":"changelog.html#v2-10-0-2017-07-21"},{"type":"extras","title":"v2.9.3 (2016-12-24) - Changelog","doc":"* Bug fixes\n  * Once again support any name for atom forms","ref":"changelog.html#v2-9-3-2016-12-24"},{"type":"extras","title":"v2.9.2 (2016-12-24) - Changelog","doc":"* Bug fixes\n  * Always read from `form.params` and then from `:selected` in `select` and `multiple_select` before falling back to `input_value/2`","ref":"changelog.html#v2-9-2-2016-12-24"},{"type":"extras","title":"v2.9.1 (2016-12-20) - Changelog","doc":"* Bug fixes\n  * Implement proper `input_value/3` callback","ref":"changelog.html#v2-9-1-2016-12-20"},{"type":"extras","title":"v2.9.0 (2016-12-19) - Changelog","doc":"* Enhancements\n  * Add `img_tag/2` helper to `Phoenix.HTML.Tag`\n  * Submit nearest form even if not direct descendent\n  * Use more iodata for `tag/2` and `content_tag/3`\n  * Add `input_value/3`, `input_id/2` and `input_name/2` as a unified API around the input (alongside `input_type/3` and `input_validations/2`)","ref":"changelog.html#v2-9-0-2016-12-19"},{"type":"extras","title":"v2.8.0 (2016-11-15) - Changelog","doc":"* Enhancements\n  * Add `csrf_meta_tag/0` helper to `Phoenix.HTML.Tag`\n  * Allow passing a `do:` option to `Phoenix.HTML.Link.button/2`","ref":"changelog.html#v2-8-0-2016-11-15"},{"type":"extras","title":"v2.7.0 (2016-09-21) - Changelog","doc":"* Enhancements\n  * Render button tags for form submits and in the `button/2` function\n  * Allow `submit/2` and `button/2` to receive `do` blocks\n  * Support the `:multiple` option in `file_input/3`\n  * Remove previously deprecated and unused `model` field","ref":"changelog.html#v2-7-0-2016-09-21"},{"type":"extras","title":"v2.6.1 (2016-07-08) - Changelog","doc":"* Enhancements\n  * Remove warnings on v1.4\n\n* Bug fixes\n  * Ensure some contents are properly escaped as an integer\n  * Ensure JavaScript data-submit events bubble up until it finds the proper parent","ref":"changelog.html#v2-6-1-2016-07-08"},{"type":"extras","title":"v2.6.0 (2016-06-16) - Changelog","doc":"* Enhancements\n  * Raise helpful error when using invalid iodata\n  * Inline date/time API with Elixir v1.3 Calendar types\n  * Add `:insert_brs` option to `text_to_html/2`\n  * Run on Erlang 19 without warnings\n\n* Client-side changes\n  * Use event delegation in `phoenix_html.js`\n  * Drop IE8 support on `phoenix_html.js`\n\n* Backwards incompatible changes\n  * `:min`, `:sec` option in `Phoenix.HTML.Form` (`datetime_select/3` and `time_select/3`) are no longer supported. Use `:minute` or `:second` instead.","ref":"changelog.html#v2-6-0-2016-06-16"},{"type":"extras","title":"v2.5.1 (2016-03-12) - Changelog","doc":"* Bug fixes\n  * Ensure multipart files work with inputs_for","ref":"changelog.html#v2-5-1-2016-03-12"},{"type":"extras","title":"v2.5.0 (2016-01-28) - Changelog","doc":"* Enhancements\n  * Introduce `form.data` field instead of `form.model`. Currently those values are kept in sync then the form is built but `form.model` will be deprecated in the long term","ref":"changelog.html#v2-5-0-2016-01-28"},{"type":"extras","title":"v2.4.0 (2016-01-21) - Changelog","doc":"* Enhancements\n  * Add `rel=nofollow` auto generation for non-get links\n  * Introduce `:selected` option for `select`  and `multiple_select`\n\n* Bug fixes\n  * Fix safe engine incorrectly marking safe code as unsafe when last expression is ` `","ref":"changelog.html#v2-4-0-2016-01-21"},{"type":"extras","title":"v2.3.0 (2015-12-16) - Changelog","doc":"* Enhancements\n  * Add `escape_javascript/1`\n  * Add helpful error message when using unknown `@inner` assign\n  * Add `Phoenix.HTML.Format.text_to_html/2`","ref":"changelog.html#v2-3-0-2015-12-16"},{"type":"extras","title":"v2.2.0 (2015-09-01) - Changelog","doc":"* Bug fix\n  * Allow the `:name` to be given in forms. For this, using `:name` to configure the underlying input name prefix has been deprecated in favor of `:as`","ref":"changelog.html#v2-2-0-2015-09-01"},{"type":"extras","title":"v2.1.2 (2015-08-22) - Changelog","doc":"* Bug fix\n  * Do not include values in `password_input/3`","ref":"changelog.html#v2-1-2-2015-08-22"},{"type":"extras","title":"v2.1.1 (2015-08-15) - Changelog","doc":"* Enhancements\n  * Allow nil in `raw/1`\n  * Allow block options in `label/3`\n  * Introduce `:skip_deleted` in `inputs_for/4`","ref":"changelog.html#v2-1-1-2015-08-15"},{"type":"extras","title":"v2.1.0 (2015-08-06) - Changelog","doc":"* Enhancements\n  * Add an index field to forms to be used by `inputs_for/4` collections","ref":"changelog.html#v2-1-0-2015-08-06"},{"type":"extras","title":"v2.0.1 (2015-07-31) - Changelog","doc":"* Bug fix\n  * Include web directory in Hex package","ref":"changelog.html#v2-0-1-2015-07-31"},{"type":"extras","title":"v2.0.0 (2015-07-30) - Changelog","doc":"* Enhancements\n  * No longer generate onclick attributes.\n\n    The main motivation for this is to provide support\n    for Content Security Policy, which recommends\n    disabling all inline scripts in a page.\n\n    We took the opportunity to also add support for\n    data-confirm in `link/2`.","ref":"changelog.html#v2-0-0-2015-07-30"},{"type":"extras","title":"v1.4.0 (2015-07-26) - Changelog","doc":"* Enhancements\n  * Support `input_type/2` and `input_validations/2` as reflection mechanisms","ref":"changelog.html#v1-4-0-2015-07-26"},{"type":"extras","title":"v1.3.0 (2015-07-23) - Changelog","doc":"* Enhancements\n  * Add `Phoenix.HTML.Form.inputs_for/4` support\n  * Add multiple select support\n  * Add reset input\n  * Infer default text context for labels","ref":"changelog.html#v1-3-0-2015-07-23"},{"type":"extras","title":"v1.2.1 (2015-06-02) - Changelog","doc":"* Bug fix\n  * Ensure nil parameters are not discarded when rendering input","ref":"changelog.html#v1-2-1-2015-06-02"},{"type":"extras","title":"v1.2.0 (2015-05-30) - Changelog","doc":"* Enhancements\n  * Add `label/3` for generating a label tag within a form","ref":"changelog.html#v1-2-0-2015-05-30"},{"type":"extras","title":"v1.1.0 (2015-05-20) - Changelog","doc":"* Enhancements\n  * Allow do/end syntax with `link/2`\n  * Raise on missing assigns","ref":"changelog.html#v1-1-0-2015-05-20"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"* Bug fixes\n  * Avoid variable clash in Phoenix.HTML engine buffers","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"* Enhancements\n  * Provides an EEx engine with HTML safe rendering\n  * Provides a `Phoenix.HTML.Safe` protocol\n  * Provides a `Phoenix.HTML.FormData` protocol\n  * Provides functions for generating tags, links and form builders in a safe way","ref":"changelog.html#v1-0-0"}],"content_type":"text/markdown"}